﻿Imports System
Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.Globalization
Imports Tsql

''' -----------------------------------------------------------------------------
''' <summary>
''' Provides functions manipulating a list of field names.
''' </summary>
''' <remarks>
''' </remarks>
''' -----------------------------------------------------------------------------
<Serializable()> _
Public Class FieldCollection
    Inherits Collection(Of String)

#Region " Public Constructors "
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Creates a <c>FieldCollection</c> object.
    ''' </summary>
    ''' -----------------------------------------------------------------------------
    Public Sub New()
        MyBase.New()
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Creates a <c>FieldCollection</c> object from a list of field names. 
    ''' </summary>
    ''' <param name="strFieldNames">A list of comma delimited field names.</param>
    ''' -----------------------------------------------------------------------------
    Public Sub New(ByVal strFieldNames As String)
        MyBase.new()
        Me.Add(strFieldNames, ","c)
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Creates a <c>FieldCollection</c> object from a list of field names. 
    ''' </summary>
    ''' <param name="strFieldNames">A list of field names.</param>
    ''' <param name="chrDelimiter">The character that separates the field names
    ''' in <paramref name="strFieldNames"/>.</param>
    ''' -----------------------------------------------------------------------------
    Public Sub New(ByVal strFieldNames As String, ByVal chrDelimiter As Char)
        MyBase.New()
        Me.Add(strFieldNames, chrDelimiter)
    End Sub
#End Region

#Region " Public Methods "
    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="strFieldNames"></param>
    ''' <remarks>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Shadows Sub Add(ByVal strFieldNames As String)
        Add(strFieldNames, ","c)
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="strFieldNames"></param>
    ''' <param name="chrDelimiter"></param>
    ''' <remarks>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Shadows Sub Add(ByVal strFieldNames As String, ByVal chrDelimiter As Char)

        If IsNothing(strFieldNames) Then
            Throw New ArgumentNullException("strFieldNames")
        End If

        Dim aFieldNames() As String = strFieldNames.Split(chrDelimiter)

        Dim nItem As Integer
        For nItem = 0 To UBound(aFieldNames)
            Dim strFieldName As String = aFieldNames(nItem).Trim
            MyBase.Add(strFieldName)
        Next nItem

    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="strFieldNames"></param>
    ''' <remarks></remarks>
    Public Sub Parse(ByVal strFieldNames As String)
        MyBase.Clear()
        Me.Add(strFieldNames, ","c)
    End Sub

    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="strFieldNames"></param>
    ''' <param name="chrDelimiter"></param>
    ''' <remarks></remarks>
    Public Sub Parse(ByVal strFieldNames As String, ByVal chrDelimiter As Char)
        MyBase.Clear()
        Me.Add(strFieldNames, chrDelimiter)
    End Sub

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets a <c>String</c> representation of a <c>FieldCollection</c>.
    ''' </summary>
    ''' <returns>A <c>String</c> representation of the <c>FieldCollection</c>.</returns>
    ''' <remarks>This method returns a human-readable string.
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Overloads Overrides Function ToString() As String
        Return ToString(","c)
    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets a <c>String</c> representation of a <c>FieldCollection</c>.
    ''' </summary>
    ''' <param name="chrDelimiter">A <c>Char</c> used to separate field names.
    ''' </param>
    ''' <returns>A <c>String</c> representation of the <c>FieldCollection</c> in 
    ''' which field names are delimited by <paramref name="chrDelimiter"/>.</returns>
    ''' <remarks>This method returns a human-readable string.
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Overloads Function ToString(ByVal chrDelimiter As Char) As String

        Dim strOutput As String = ""
        Dim nField As Integer

        For nField = 0 To Me.Count - 1
            If (nField > 0) Then
                strOutput = String.Concat(strOutput, chrDelimiter)
            End If
            strOutput = String.Concat(strOutput, Me.Item(nField))
        Next
        Return strOutput
    End Function

    Public Overloads Function ToSql() As String
        Return Me.ToSql(","c, """"c, """"c)
    End Function

    Public Overloads Function ToSql(ByVal strPrefix As String) As String
        Return Me.ToSql(strPrefix, ","c, """"c, """"c)
    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets a <c>String</c> representation of a <c>FieldCollection</c>.
    ''' </summary>
    ''' <param name="chrDelimiter">A <c>Char</c> used to separate field names.
    ''' </param>
    ''' <param name="chrOpenQuote">A <c>Char</c> that is placed around field names.
    ''' </param>
    ''' <param name="chrCloseQuote">A <c>Char</c> that is placed around field names.
    ''' </param>
    ''' <returns>A <c>String</c> representation of the <c>FieldCollection</c> in 
    ''' which field names are delimited by <paramref name="chrDelimiter" /> and
    ''' enclosed in <paramref name="chrQuote" /> characters.</returns>
    ''' <remarks>This method returns a human-readable string.
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Overloads Function ToSql(ByVal chrDelimiter As Char, ByVal chrOpenQuote As Char, ByVal chrCloseQuote As Char) As String
        Dim strOutput As String = ""
        Dim nField As Integer

        For nField = 0 To Me.Count - 1
            If (nField > 0) Then
                strOutput = String.Concat(strOutput, chrDelimiter)
            End If
            strOutput = String.Concat(strOutput, chrOpenQuote, Me.Item(nField), chrCloseQuote)
        Next
        Return strOutput
    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets a <c>String</c> representation of a <c>FieldCollection</c>.
    ''' </summary>
    ''' <param name="strPrefix">A <c>String</c> to prefix field names with.</param>
    ''' <param name="chrDelimiter">A <c>Char</c> used to separate field names.
    ''' </param>
    ''' <param name="chrOpenQuote">A <c>Char</c> that is placed around field names.
    ''' </param>
    ''' <param name="chrCloseQuote">A <c>Char</c> that is placed around field names.
    ''' </param>
    ''' <returns>A <c>String</c> representation of the <c>FieldCollection</c> in 
    ''' which field names are delimited by <paramref name="chrDelimiter" />,
    ''' prefixed with <paramref name="strPrefix"/> and enclosed in <paramref 
    ''' name="chrQuote" /> characters.</returns>
    ''' <remarks>This method returns a human-readable string.
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Overloads Function ToSql(ByVal strPrefix As String, ByVal chrDelimiter As Char, ByVal chrOpenQuote As Char, ByVal chrCloseQuote As Char) As String
        Dim strOutput As String = ""
        Dim nField As Integer

        For nField = 0 To Me.Count - 1
            If (nField > 0) Then
                strOutput = String.Concat(strOutput, chrDelimiter)
            End If
            strOutput = String.Concat(strOutput, chrOpenQuote, strPrefix, Me.Item(nField), chrCloseQuote)
        Next
        Return strOutput
    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets an SQL alias <c>String</c> for a <c>FieldCollection</c>.
    ''' </summary>
    ''' <param name="strAliasPrefix">A <c>String</c> to prefix field names with.
    ''' </param>
    ''' <returns>An SQL alias <c>String</c> that can be used to select fields from 
    ''' a table and return a resultset in which field names are prefixed with 
    ''' <paramref name="strAliasPrefix" />.</returns>
    ''' <remarks>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Overloads Function ToSqlAliasString(ByVal strAliasPrefix As String) As String
        Return ToSqlAliasString(Keyword.FieldDelimiter, Keyword.OpenQuote, Keyword.CloseQuote, strAliasPrefix)
    End Function

    ''' -----------------------------------------------------------------------------
    ''' <summary>
    ''' Gets an SQL alias <c>String</c> for a <c>FieldCollection</c>.
    ''' </summary>
    ''' <param name="chrDelimiter">A <c>Char</c> used to separate field names.
    ''' </param>
    ''' <param name="chrOpenQuote">A <c>Char</c> that is placed around field names.
    ''' </param>
    ''' <param name="chrCloseQuote">A <c>Char</c> that is placed around field names.
    ''' </param>
    ''' <param name="strAliasPrefix">A <c>String</c> to prefix field names with.
    ''' </param>
    ''' <returns>An SQL alias <c>String</c> that can be used to select fields from 
    ''' a table and return a resultset in which field names are prefixed with 
    ''' <paramref name="strAliasPrefix" />.</returns>
    ''' <remarks>
    ''' </remarks>
    ''' -----------------------------------------------------------------------------
    Public Overloads Function ToSqlAliasString(ByVal chrDelimiter As Char, ByVal chrOpenQuote As Char, ByVal chrCloseQuote As Char, ByVal strAliasPrefix As String) As String
        Dim strOutput As String = ""
        Dim nField As Integer

        For nField = 0 To Me.Count - 1
            If (nField > 0) Then
                strOutput = String.Concat(strOutput, chrDelimiter)
            End If

            Dim strField As String = String.Format("{0}{2}{1}{3}{0}{4}{2}{1}", chrOpenQuote, chrCloseQuote, Item(nField), Keyword.As, strAliasPrefix)
            strOutput = String.Concat(strOutput, strField)
        Next
        Return strOutput
    End Function
#End Region

End Class
