﻿#Region " Imports "
Imports Infragistics.Web.UI.GridControls
Imports Tsql

#End Region

''' <summary>
''' Class GridGroupedColumn
''' </summary>
<Serializable()>
Public Class GridGroupedColumn

#Region " Declarations "
    Private _name As String
    Private _sortDirection As Infragistics.Web.UI.GridControls.GroupingSortDirection
#End Region

#Region " Constructors "
    ''' <summary>
    ''' Initializes a new instance of the <see cref="GridGroupedColumn"/> class.
    ''' </summary>
    Public Sub New()
        MyBase.New()
    End Sub

    ''' <summary>
    ''' Initializes a new instance of the <see cref="GridGroupedColumn"/> class.
    ''' </summary>
    ''' <param name="groupBy">The group by criteria string.</param>
    Public Sub New(ByVal groupBy As String)
        MyBase.New()

        If (groupBy.EndsWith(" (ascending)")) Then
            Me._name = Field.GetFieldName(groupBy.Substring(0, groupBy.Length - 12))
            Me._sortDirection = Infragistics.Web.UI.GridControls.GroupingSortDirection.Ascending
        ElseIf (groupBy.EndsWith(" ASC")) Then
            Me._name = Field.GetFieldName(groupBy.Substring(0, groupBy.Length - 4))
            Me._sortDirection = Infragistics.Web.UI.GridControls.GroupingSortDirection.Ascending
        ElseIf (groupBy.EndsWith(" ASCENDING")) Then
            Me._name = Field.GetFieldName(groupBy.Substring(0, groupBy.Length - 10))
            Me._sortDirection = Infragistics.Web.UI.GridControls.GroupingSortDirection.Ascending
        ElseIf (groupBy.EndsWith(" (descending)")) Then
            Me._name = Field.GetFieldName(groupBy.Substring(0, groupBy.Length - 13))
            Me._sortDirection = Infragistics.Web.UI.GridControls.GroupingSortDirection.Descending
        ElseIf (groupBy.EndsWith(" DESC")) Then
            Me._name = Field.GetFieldName(groupBy.Substring(0, groupBy.Length - 5))
            Me._sortDirection = Infragistics.Web.UI.GridControls.GroupingSortDirection.Descending
        ElseIf (groupBy.EndsWith(" DESCENDING")) Then
            Me._name = Field.GetFieldName(groupBy.Substring(0, groupBy.Length - 11))
            Me._sortDirection = Infragistics.Web.UI.GridControls.GroupingSortDirection.Descending
        Else
            Me._name = Field.GetFieldName(groupBy)
            Me._sortDirection = Infragistics.Web.UI.GridControls.GroupingSortDirection.Ascending
        End If

    End Sub

    ''' <summary>
    ''' Initializes a new instance of the <see cref="GridGroupedColumn"/> class.
    ''' </summary>
    ''' <param name="groupedColumn">The grouped column.</param>
    Public Sub New(ByVal groupedColumn As GroupedColumn)
        MyBase.New()

        Me._name = groupedColumn.ColumnKey
        Me._sortDirection = groupedColumn.SortDirection

    End Sub
#End Region

#Region " Properties "
    ''' <summary>
    ''' Gets or sets the name.
    ''' </summary>
    ''' <value>The name.</value>
    Public Property Name() As String
        Get
            Return Me._name
        End Get
        Set(value As String)
            Me._name = value
        End Set
    End Property

    ''' <summary>
    ''' Gets or sets the sort direction.
    ''' </summary>
    ''' <value>The sort direction.</value>
    Public Property SortDirection() As Infragistics.Web.UI.GridControls.GroupingSortDirection
        Get
            Return Me._sortDirection
        End Get
        Set(value As Infragistics.Web.UI.GridControls.GroupingSortDirection)
            Me._sortDirection = value
        End Set
    End Property
#End Region

#Region " Overrides "
    ''' <summary>
    ''' Returns a <see cref="System.String" /> that represents this instance.
    ''' </summary>
    ''' <returns>A <see cref="System.String" /> that represents this instance.</returns>
    Public Overrides Function ToString() As String

        If (String.IsNullOrWhiteSpace(_name)) Then
            Return String.Empty
        End If

        Return String.Format("{0} ({1})", _name, SortDirection.ToString().ToLower())

    End Function

    ''' <summary>
    ''' Returns an SQL group criteria string from a <c>GridGroupedColumn</c> object.
    ''' </summary>
    Public Overridable Function ToSql() As String

        If (String.IsNullOrWhiteSpace(_name)) Then
            Return String.Empty
        End If

        Return String.Format("{0}{1}", Field.GetQuotedFieldName(_name), IIf(SortDirection.Equals(GroupingSortDirection.Ascending), Keyword.Asc, Keyword.Desc))

    End Function
#End Region

End Class
